@@ -158,6 +158,9 @@ struct flb_input_instance {
     /* By default all input instances are 'routable' */
     int routable;
 
+    /* flag to pause input when storage is full */
+    int storage_pause_on_chunks_overlimit;
+
     /*
      * Input network info:
      *
@@ -210,6 +213,14 @@ struct flb_input_instance {
      */
     int mem_buf_status;
 
+    /*
+     * Define the buffer status:
+     *
+     * - FLB_INPUT_RUNNING -> can append more data
+     * - FLB_INPUT_PAUSED  -> cannot append data
+     */
+    int storage_buf_status;
+
     /*
      * Optional data passed to the plugin, this info is useful when
      * running Fluent Bit in library mode and the target plugin needs
@@ -495,6 +506,9 @@ static inline int flb_input_buf_paused(struct flb_input_instance *i)
     if (i->mem_buf_status == FLB_INPUT_PAUSED) {
         return FLB_TRUE;
     }
+    if (i->storage_buf_status == FLB_INPUT_PAUSED) {
+        return FLB_TRUE;
+    }
 
     return FLB_FALSE;
 }
