@@ -41,7 +41,8 @@ struct cio_file {
 #endif
     /* cached addr */
     char *st_content;
-    crc_t crc_cur;
+    crc_t crc_cur;            /* crc: current value calculated */
+    int crc_reset;            /* crc: must recalculate from the beginning ? */
 };
 
 size_t cio_file_real_size(struct cio_file *cf);
