@@ -153,6 +153,51 @@ static int get_machine_id(struct flb_calyptia *ctx, char **out_buf, size_t *out_
     return 0;
 }
 
+static void pack_str(msgpack_packer *mp_pck, char *str)
+{
+    int len;
+
+    len = strlen(str);
+    msgpack_pack_str(mp_pck, len);
+    msgpack_pack_str_body(mp_pck, str, len);
+}
+
+static void pack_env(struct flb_env *env, char *key, struct flb_mp_map_header *h,
+                     msgpack_packer *mp_pck)
+{
+    char *val;
+
+    val = (char *) flb_env_get(env, key);
+    if (val) {
+        flb_mp_map_header_append(h);
+        pack_str(mp_pck, key);
+        pack_str(mp_pck, val);
+    }
+}
+
+static void pack_env_metadata(struct flb_env *env,
+                              struct flb_mp_map_header *mh, msgpack_packer *mp_pck)
+{
+    char *tmp;
+    struct flb_mp_map_header h;
+
+    /* Kubernetes */
+    tmp = (char *) flb_env_get(env, "k8s");
+    if (tmp && strcasecmp(tmp, "enabled") == 0) {
+        flb_mp_map_header_append(mh);
+        pack_str(mp_pck, "k8s");
+
+        /* adding k8s map */
+        flb_mp_map_header_init(&h, mp_pck);
+
+        pack_env(env, "k8s.namespace", &h, mp_pck);
+        pack_env(env, "k8s.pod_name", &h, mp_pck);
+        pack_env(env, "k8s.node_name", &h, mp_pck);
+
+        flb_mp_map_header_end(&h);
+    }
+}
+
 static flb_sds_t get_agent_metadata(struct flb_calyptia *ctx)
 {
     int len;
@@ -162,6 +207,7 @@ static flb_sds_t get_agent_metadata(struct flb_calyptia *ctx)
     struct flb_mp_map_header mh;
     msgpack_sbuffer mp_sbuf;
     msgpack_packer mp_pck;
+    struct flb_config *config = ctx->config;
 
     /* init msgpack */
     msgpack_sbuffer_init(&mp_sbuf);
@@ -225,6 +271,9 @@ static flb_sds_t get_agent_metadata(struct flb_calyptia *ctx)
     msgpack_pack_str(&mp_pck, len);
     msgpack_pack_str_body(&mp_pck, ctx->machine_id, len);
 
+    /* pack environment metadata */
+    pack_env_metadata(config->env, &mh, &mp_pck);
+
     /* finalize */
     flb_mp_map_header_end(&mh);
 
@@ -657,12 +706,16 @@ static struct flb_calyptia *config_init(struct flb_output_instance *ins,
         }
     }
 
-    /* machine id */
-    ret = get_machine_id(ctx, &machine_id, &size);
-    if (ret == -1) {
-        return NULL;
+    /* If no machine_id has been provided via a configuration option get it from the local machine-id. */
+    if (!ctx->machine_id) {
+        /* machine id */
+        ret = get_machine_id(ctx, &machine_id, &size);
+        if (ret == -1) {
+            return NULL;
+        }
+        ctx->machine_id = (flb_sds_t) machine_id;
     }
-    ctx->machine_id = (flb_sds_t) machine_id;
+
     flb_plg_debug(ctx->ins, "machine_id=%s", ctx->machine_id);
 
     /* Upstream */
@@ -774,6 +827,7 @@ static void cb_calyptia_flush(const void *data, size_t bytes,
             flb_upstream_conn_release(u_conn);
             FLB_OUTPUT_RETURN(FLB_ERROR);
         }
+        cmt_destroy(cmt);
     }
     else {
         out_buf = (char *) data;
@@ -845,6 +899,8 @@ static int cb_calyptia_exit(void *data, struct flb_config *config)
     if (ctx->fs) {
         flb_fstore_destroy(ctx->fs);
     }
+
+    flb_kv_release(&ctx->kv_labels);
     flb_free(ctx);
 
     return 0;
@@ -869,6 +925,11 @@ static struct flb_config_map config_map[] = {
      0, FLB_TRUE, offsetof(struct flb_calyptia, api_key),
      "Calyptia Cloud API Key."
     },
+    {
+     FLB_CONFIG_MAP_STR, "machine_id", NULL,
+     0, FLB_TRUE, offsetof(struct flb_calyptia, machine_id),
+     "Custom machine_id to be used when registering agent"
+    },
 
     {
      FLB_CONFIG_MAP_STR, "store_path", NULL,
@@ -876,6 +937,12 @@ static struct flb_config_map config_map[] = {
      ""
     },
 
+    {
+     FLB_CONFIG_MAP_SLIST_1, "add_label", NULL,
+     FLB_CONFIG_MAP_MULT, FLB_TRUE, offsetof(struct flb_calyptia, add_labels),
+     "Label to append to the generated metric."
+    },
+
     /* EOF */
     {0}
 };
