@@ -25,6 +25,7 @@
 #include "metrics.h"
 #include "storage.h"
 #include "plugins.h"
+#include "health.h"
 
 int api_v1_registration(struct flb_hs *hs)
 {
@@ -32,6 +33,10 @@ int api_v1_registration(struct flb_hs *hs)
     api_v1_metrics(hs);
     api_v1_plugins(hs);
 
+    if (hs->config->health_check == FLB_TRUE) {
+        api_v1_health(hs);
+    }
+
     if (hs->config->storage_metrics == FLB_TRUE) {
         api_v1_storage_metrics(hs);
     }
